#!/bin/sh

SKIP_XKB_CZSK_PATCH=0
SKIP_XKB_XFREE86_PATCH=0
SKIP_XKB_GROUP_LED_PATCH=0
SKIP_XKEYSYM_PATCH=0
SKIP_XLOCALE_PATCH=0
SKIP_LIBC5=0
OLDLIBS_DIR=""

IS_LIBC6_SYSTEM=0
IS_LIBC5_SYSTEM=0

LC_ALL=C ; export LC_ALL
LANGUAGE=C ; export LANGUAGE

LOCALEDIR=`localedef --help | sed -ne '/locale files/s/^[^/]*\(\/[^:]*\).*$/\1/p' | head -1`
LOCALES="cs cz cs_CZ sk sk_SK"
TMPTARFILE=/tmp/forcedIM.$$.tar

echo "Checking configuration..."
if [ ! `whoami` = root ]
then
	echo "Sorry `whoami`, you must be root to perform this installation"
	exit 1
fi

if grep translations /usr/X11R6/lib/X11/app-defaults/XTerm 2> /dev/null | grep 'keymap(' > /dev/null 2>&1
then
	echo "It seems you have installed a national keyboard through xterm translations;"
	echo "please replace your /usr/X11R6/lib/X11/app-defaults/XTerm by the original"
	echo "version"
	exit 1
fi

if ! which X > /dev/null 2>&1
then
	echo "Oops - cannot find X in your path - which system are you running?"
	exit 1
fi

if [ -z "$LOCALEDIR" ]
then
	echo "Cannot determine your locale - check output of localedef --help"
	exit 1
fi

if ldd `which X` | grep 'libc.so.6' > /dev/null 2>&1
then
	IS_LIBC6_SYSTEM=1
	echo 'You are running a libc6 (glibc) system'
elif ldd `which X` | grep 'libc.so.5' > /dev/null 2>&1
then
	IS_LIBC5_SYSTEM=1
	echo "You are running a libc5 system"
else
	echo "Oops - your X executable is neither libc.so.5 nor"
	echo "libc.so.6-linked - which system are you running?"
	exit 1
fi

	
if [ ! -e $LOCALEDIR ]
then
	echo "Your don't have $LOCALEDIR directory"
	exit 1
fi

echo "Your locale directory is $LOCALEDIR"

if [ -f /usr/X11R6/lib/X11/xkb/symbols/czsk ]
then
	echo "The CZ and SK keymaps appear to be installed on this system."
	echo "Skiping them - please, check this and eventually copy it manually"
	echo ""
	SKIP_XKB_CZSK_PATCH=1
fi

if grep sk_us_qwertz /usr/X11R6/lib/X11/xkb/keymap/xfree86 > /dev/null 2>&1
then
	echo "The CZ and SK keymap entries appear to be installed on this system."
	echo "Skipping it - please, check this and eventually add xfree86.add manually"
	echo ""
	SKIP_XKB_CZSK_PATCH=1
	SKIP_XKB_XFREE86_PATCH=1
fi

if [ -f /usr/X11R6/lib/X11/xkb/compat/group_led ]
then
	echo "The group_led file appears to be installed on this system."
	echo "Skiping them - please, check this and eventually copy it manually"
	echo ""
	SKIP_XKB_GROUP_LED_PATCH=1
fi

if grep il2_ /usr/X11R6/lib/X11/XKeysymDB > /dev/null 2>&1
then
	echo "The keysym patch appears to be installed on this system."
	echo "Skipping it - please, check this and eventually add XKeysymDB.add manually"
	echo ""
	SKIP_XKEYSYM_PATCH=1
fi

if grep 'ct_encoding.*1:GR' /usr/X11R6/lib/X11/locale/iso8859-2/XLC_LOCALE > /dev/null 2>&1
then
	echo "The XLC_LOCALE patch appears to be installed on this system."
	echo "Skipping it - please, check this and eventually apply manually"
	echo ""
	SKIP_XLOCALE_PATCH=1
fi


if ls /usr/i486-linuxlibc5/lib/libX11.so.6.1 > /dev/null 2>&1
then
	OLDLIBS_DIR=/usr/i486-linuxlibc5/lib
elif ls /usr/i486-linux-libc5/lib/libX11.so.6.1 > /dev/null 2>&1
then
	OLDLIBS_DIR=/usr/i486-linux-libc5/lib
else
	echo 'I have not found your old (libc5) X libraries.'
	echo "Skipping installation of old library - check this and eventually copy manually"
	SKIP_LIBC5=1
fi

(
	echo "Preserving old contents of $LOCALEDIR..."
	rm -f $TMPTARFILE
	cd $LOCALEDIR
	for f in $LOCALES
	do
		if [ -d $LOCALEDIR/$f ]
		then
			tar uhf $TMPTARFILE $f
		fi
	done

	echo "Installing/updating locales..."
	rm -rf $LOCALES

	mkdir cs_CZ sk_SK
	chown root.root cs_CZ sk_SK
	chmod 755 cs_CZ sk_SK
	ln -s cs_CZ cs
	ln -s cs_CZ cz
	ln -s sk_SK sk
)

cat cs_CZ.tar.gz | ( cd $LOCALEDIR ; tar xfz - ) 
cat sk_SK.tar.gz | ( cd $LOCALEDIR ; tar xfz - ) 

if [ -f $TMPTARFILE ]
then
	( cd $LOCALEDIR ; tar xf $TMPTARFILE )
	rm -f $TMPTARFILE
fi

if ! locale -a | grep ^cs$ > /dev/null 2>&1
then
	echo "Installation of czech locale failed - check manually"
fi

if ! locale -a | grep ^sk$ > /dev/null 2>&1
then
	echo "Installation of slovak locale failed - check manually"
fi

if [ $SKIP_XKB_CZSK_PATCH = 0 ]
then
	echo "Installing czech and slovak kbd maps ..."

	if cp -f czsk /usr/X11R6/lib/X11/xkb/symbols/czsk
	then
		chown root.root /usr/X11R6/lib/X11/xkb/symbols/czsk
		chmod 644 /usr/X11R6/lib/X11/xkb/symbols/czsk
		echo Done.
	else
		echo "Something is wrong -- patch failed. Is XKB installed here?"
	fi
fi

if [ $SKIP_XKB_XFREE86_PATCH = 0 ]
then
	echo "Patching xfree86 file..."
	if cat xfree86.add >> /usr/X11R6/lib/X11/xkb/keymap/xfree86
	then
		echo Done.
	else
		echo "Something is wrong -- patch failed."
	fi
fi

if [ $SKIP_XKB_GROUP_LED_PATCH = 0 ]
then
	echo "Installing group_led file ..."

	if cp -f group_led /usr/X11R6/lib/X11/xkb/compat/group_led
	then
		chown root.root /usr/X11R6/lib/X11/xkb/compat/group_led
		chmod 644 /usr/X11R6/lib/X11/xkb/compat/group_led
		echo Done.
	else
		echo "Something is wrong -- patch failed. Is XKB installed here?"
	fi
fi

if [ $SKIP_XKEYSYM_PATCH = 0 ]
then
	echo "Patching XKeysymDB file..."
	if cat XKeysymDB.add >> /usr/X11R6/lib/X11/XKeysymDB
	then
		echo Done.
	else
		echo "Something is wrong -- patch failed."
	fi
fi

if [ $SKIP_XLOCALE_PATCH = 0 ]
then
	echo "Patching XLC_LOCALE file..."
	ed /usr/X11R6/lib/X11/locale/iso8859-2/XLC_LOCALE > /dev/null 2>&1 << EOF
,s/ct_encoding.*:GR/ct_encoding	ISO8859-2:GR; ISO8859-1:GR/
w
q
EOF

	if [ $? = 0 ]
	then
		echo Done.
	else
		echo "Something is wrong -- patch failed."
	fi
fi

echo "Recompiling .dir files..."
( cd /usr/X11R6/lib/X11/xkb/compat ; /usr/X11R6/bin/xkbcomp -llphfR '*' -o /usr/X11R6/lib/X11/xkb/compat.dir )
( cd /usr/X11R6/lib/X11/xkb/keymap ; /usr/X11R6/bin/xkbcomp -llphfR '*' -o /usr/X11R6/lib/X11/xkb/keymap.dir )
echo Done.

if [ $IS_LIBC6_SYSTEM = 1 ]
then
	echo 'Installing patched libX11 (glibc version)...' 
	cp -pf libX11-glibc.so.6.99 /usr/X11R6/lib/libX11.so.6.99
	chmod 755 /usr/X11R6/lib/libX11.so.6.99
	chown root.root /usr/X11R6/lib/libX11.so.6.99
	echo Done.

	if [ $SKIP_LIBC5 = 0 ]
	then
		echo 'Installing patched libX11 (libc5 version)...'
		cp -pf libX11-libc5.so.6.99 $OLDLIBS_DIR/libX11.so.6.99
		chmod 755 $OLDLIBS_DIR/libX11.so.6.99
		chown root.root $OLDLIBS_DIR/libX11.so.6.99
		echo Done.
	fi
elif [ $IS_LIBC5_SYSTEM = 1 ]
then
	echo 'Installing patched libX11 (libc5 version)...'
	cp -pf libX11-libc5.so.6.99 /usr/X11R6/lib/libX11.so.6.99
	chmod 755 /usr/X11R6/lib/libX11.so.6.99
	chown root.root /usr/X11R6/lib/libX11.so.6.99
	echo Done.
fi

echo "Rebuilding dynamic linker data..."
ldconfig
echo Done.

cat <<EOF
*****************************************************************
*****************************************************************
Installation was successful.

Read the README file for further steps required:

- XF86Config changes
- locale environment variables
- forced i18n environment variable

Installation finished.
EOF
exit 0
